/* --------------------------------------------------------------------------
 * BreakOut
 * --------------------------------------------------------------------------
 * prog:  Max Rheiner und Simon Gwinner
 * date:  26/09/2012 (m/d/y)
 * ----------------------------------------------------------------------------
 */

import ddf.minim.*;

Minim minim;
AudioSample ballHitSnd;
AudioSample ballHitSchlaegerSnd;
AudioSample gameoverSnd;

BouncingBall ball;
Racket       schlaeger = null;
Brick[]      brickList = null;

final static int GAME_MODE_MENU      = 0;
final static int GAME_MODE_PLAY      = 1;
final static int GAME_MODE_GAMEOVER  = 2;

int          gameMode = GAME_MODE_MENU;
PFont        f;
PShape      _waterback;
PShape      _waterfront;
PShape      _background;
PImage      _menu;
PImage      _gameover;
void setup()
{
  size(800, 600); 

  ball = new BouncingBall(width/2, height/2,15);
  schlaeger = new Racket(0, height - 78, 250, 45);

  _waterback  = loadShape("water-back.svg");
  _waterfront = loadShape("water-front.svg");
  _background = loadShape("background.svg");
  _menu = loadImage("start.png");
  _gameover = loadImage("gameover.png");


  // sound init
  minim = new Minim(this);
  ballHitSnd = minim.loadSample("bird.mp3");
  ballHitSchlaegerSnd = minim.loadSample("waterdrop.mp3");
  gameoverSnd = minim.loadSample("whale.mp3");


  //font
  f = loadFont("AttitudeInline-48.vlw");
  smooth(); 
  noCursor();
}

void initGameScene()
{
  // setzte die start posititon des balles
  ball.set(new PVector(width/2, height/2), new PVector(random(-7, 7), -7), 1);
  
  // erzeuge die ziegel
  brickList = new Brick[30];
  for (int i=0;i < brickList.length;i++)
  {
    if (i<=9)
      brickList[i] = new Brick(80 + i * 70, 110, 50, 20, (int) random(0, 4));
    else if (i>=9 && i<=19)
      brickList[i] = new Brick(80 + (i-10) * 70, 170, 50, 20, (int) random(0, 4));
    else if (i>=17 && i<=29)
      brickList[i] = new Brick(80 + (i-20) * 70, 230, 50, 20, (int) random(0, 4));
  }
}


void draw()
{
  // ghosting
  fill(185, 180, 180, 100);
  rect(0, 0, width, height);

  drawScene();
}

void drawScene()
{
  switch(gameMode)
  {
  case GAME_MODE_MENU:
    drawMenu();
    break;
  case GAME_MODE_PLAY:
    drawGame();
    break;  
  case GAME_MODE_GAMEOVER:
    drawGameOver();
    break;
  }
}

void drawMenu()
{
  pushStyle();
  fill(255);
  image(_menu, -10, 70);
  popStyle();
}

void drawGameOver()
{
  pushStyle();
  fill(255);
  image(_gameover, -10, 70);
  popStyle();
}

void drawGame()
{
  shape(_background, 122, 200, width*0.7, height*0.7);
  // update die daten, berechne kollisionen
  updateScene();



  // zeichne ziegel
  for (int i=0;i < brickList.length;i++)
  {
    if (brickList[i] != null)
      brickList[i].draw();
  }

  // zeichne den ball
  ball.draw();

  // zeichne schlaeger
  shape(_waterback, -50, 285, 1000, 600);
  schlaeger.draw();
  shape(_waterfront, -50, 210, 1000, 600);
}

void keyPressed()
{
  switch(key)
  {
  case ENTER:
    // start game
    gameMode = GAME_MODE_PLAY;
    initGameScene();
    break;
  case ' ':
    // end
    gameMode = GAME_MODE_MENU;
    break;
  }

  switch(keyCode) {
  case LEFT:
    schlaeger.keycontrol -= 30;
    break;
  case RIGHT:
    schlaeger.keycontrol += 30;
    break;
  }
}

void  updateScene()
{
  // test ob ball aus dem spielfeld ist
  if (ball._pos.y > height- ball._r)
  {
    gameMode = GAME_MODE_GAMEOVER;
    // spiele sound
    gameoverSnd.trigger();
    return;
  }

  // update schlaeger pos
  schlaeger.move(mouseX);

  PVector hitPos = new PVector();
  float  hitV;
  float  speed = 0;

  // teste ob ball den schlaeger trifft
  int retVal = schlaeger.checkCollision(ball._pos.x, ball._pos.y, ball._r, ball._dir, hitPos);
  switch(retVal)
  {
  case Brick.HIT_LEFT:
    // aendere flugrichtung ball
    ball._dir.x *= -1;

    // setzte ball buendig auf den schlaeger(das er nicht im schlaeger ist)
    ball._pos.x = schlaeger._pos.x - schlaeger._w/2 - ball._r;

    // spiele sound
    ballHitSchlaegerSnd.trigger();
    break;
  case Brick.HIT_RIGTH:
    // aendere flugrichtung ball
    ball._dir.x *= -1;

    // setzte ball buendig auf den schlaeger(das er nicht im schlaeger ist)
    ball._pos.x = schlaeger._pos.x + schlaeger._w/2 + ball._r;

    // spiele sound
    ballHitSchlaegerSnd.trigger();
    break;
  case Brick.HIT_TOP:
    // aendere flugrichtung ball

    // anschnitt berechnen 
    hitV = hitPos.x - (schlaeger._pos.x - schlaeger._w/2);
    hitV = 1.0 / schlaeger._w * hitV - 0.5;

    speed = ball._dir.mag();

    ball._dir.x += hitV * 8; 
    ball._dir.y *= -1;

    // ball soll die gleiche geschwindigkeit haben wie am anfang
    ball._dir.normalize();
    ball._dir.mult(speed);

    // setzte ball buendig auf den schlaeger(das er nicht im schlaeger ist)
    ball._pos.y = schlaeger._pos.y - schlaeger._h/2 - ball._r ;

    // spiele sound
    ballHitSchlaegerSnd.trigger();
    break; 
  case Brick.HIT_BOTTOM:
    // aendere flugrichtung ball

    // anschnitt berechnen 
    hitV = hitPos.x - (schlaeger._pos.x - schlaeger._w/2);
    hitV = 1.0 / schlaeger._w * hitV - 0.5;

    speed = ball._dir.mag();

    ball._dir.x += hitV * 8; 
    ball._dir.y *= -1;

    // ball soll die gleiche geschwindigkeit haben wie am anfang
    ball._dir.normalize();
    ball._dir.mult(speed);

    // setzte ball buendig auf den schlaeger(das er nicht im schlaeger ist)
    ball._pos.y = schlaeger._pos.y + schlaeger._h/2 + ball._r ;

    // spiele sound
    ballHitSchlaegerSnd.trigger();
    break;
  }

  // test ob ball einen ziegel trifft
  for (int i=0;i < brickList.length;i++)
  {
    if (brickList[i] != null)
    {
      retVal = brickList[i].checkCollision(ball._pos.x, ball._pos.y, ball._r, ball._dir, hitPos);
      switch(retVal)
      {
      case Brick.HIT_LEFT:
      case Brick.HIT_RIGTH:
        // aendere flugrichtung ball
        ball._dir.x *= -1;

        // entferne ziegel aus liste

        brickList[i] = null;

        // spiele sound
        ballHitSnd.trigger();
        break;
      case Brick.HIT_TOP:
      case Brick.HIT_BOTTOM:

        speed = ball._dir.mag();
        // aendere flugrichtung ball
        ball._dir.y *= -1;

        // entferne ziegel aus liste
        if (brickList[i]._typ == 1)
          brickList[i] = null;
        else if (brickList[i]._typ ==2)
        {brickList[i] = null;              //geschwindigkeit ändert sich bei blauem vogel
          ball._dir.normalize();
          ball._dir.mult(5);}
        else if (brickList[i]._typ ==3)    //geschwindigkeit ändert sich bei rotem vogel
         { brickList[i] = null;
          ball._dir.normalize();
          ball._dir.mult(8);
        }


        // spiele sound
        ballHitSnd.trigger();
        break;
      }
    }
  }
}




